/*  Copyright 2010-2011 Guillaume Duhamel

    This file is part of Gophrier.

    Gophrier is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Gophrier is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Gophrier; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
*/

#include "cli.h"
#include "conf.h"

#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

static void cli_version(void)
{
    fprintf(stderr, "gophrier " GOPHRIER_VERSION "\n");
    exit(0);
}

static void cli_usage(void)
{
    fprintf(stderr, "usage: gophrier [OPTION]\n");
    fprintf(stderr, " -4       run in ipv4 mode\n");
    fprintf(stderr, " -6       run in ipv6 mode\n");
    fprintf(stderr, " -c PATH  read gophrier configuration in the specified PATH\n");
    fprintf(stderr, " -d 0|1   run as a daemon (1) or as a normal process (0)\n");
    fprintf(stderr, " -l PATH  output log files in the specified PATH\n");
    fprintf(stderr, " -p PORT  listen on PORT\n");
    fprintf(stderr, " -r PATH  use PATH as the root directory\n");
    fprintf(stderr, " -h       display this help and exit\n");
    fprintf(stderr, " -v       display version and exit\n");
    exit(6);
}

static void cli_error(const char * format, ...)
{
    va_list l;

    va_start(l, format);
    vfprintf(stderr, format, l);
    va_end(l);
    exit(6);
}

static void cli_conf(int * argc, char *** argv)
{
    (*argv)++;
    (*argc)--;
    if (*argc)
        conf_set_conf_dir(**argv);
    else
        cli_error("missing conf parameter\n");
}

static void cli_daemon(int * argc, char *** argv)
{
    (*argv)++;
    (*argc)--;
    if (*argc)
    {
        char * end;
        int value = strtol(**argv, &end, 10);
        if (end == **argv + strlen(**argv))
            conf_set_daemon(value);
        else
            cli_error("%s is not a valid value for port\n", **argv);
    }
    else
        cli_error("missing daemon parameter\n");
}

static void cli_ipv4(void)
{
    conf_set_ipv6(0);
}

static void cli_ipv6(void)
{
    conf_set_ipv6(1);
}

static void cli_log(int * argc, char *** argv)
{
    (*argv)++;
    (*argc)--;
    if (*argc)
        conf_set_log_dir(**argv);
    else
        cli_error("missing log parameter\n");
}

static void cli_port(int * argc, char *** argv)
{
    (*argv)++;
    (*argc)--;
    if (*argc)
    {
        char * end;
        int value = strtol(**argv, &end, 10);
        if (end == **argv + strlen(**argv))
            conf_set_port(value);
        else
            cli_error("%s is not a valid value for port\n", **argv);
    }
    else
        cli_error("missing port parameter\n");
}

static void cli_root(int * argc, char *** argv)
{
    (*argv)++;
    (*argc)--;
    if (*argc)
        conf_set_root_dir(**argv);
    else
        cli_error("missing root parameter\n");
}

void cli_parse(int * argc, char *** argv)
{
    (*argv)++;
    (*argc)--;
    while(*argc)
    {
        if (! strcmp("-4", **argv)) cli_ipv4();
        else if (! strcmp("-6", **argv)) cli_ipv6();
        else if (! strcmp("-c", **argv)) cli_conf(argc, argv);
        else if (! strcmp("-d", **argv)) cli_daemon(argc, argv);
        else if (! strcmp("-h", **argv)) cli_usage();
        else if (! strcmp("-l", **argv)) cli_log(argc, argv);
        else if (! strcmp("-p", **argv)) cli_port(argc, argv);
        else if (! strcmp("-r", **argv)) cli_root(argc, argv);
        else if (! strcmp("-v", **argv)) cli_version();
        else cli_error("invalid option: %s\n", **argv);
        (*argv)++;
        (*argc)--;
    }
}
